<?php
ini_set('display_errors', 1);
error_reporting(E_ALL);

include(__DIR__.'/../config.php');
include(__DIR__.'/../libraries/Version.class.php');
include(__DIR__.'/../libraries/Sandbox.class.php');

$error = array();
$message = '';
$data = array();

$mysqli = new mysqli(__DVERSIONS_MAIN_DB_HOST__, __DVERSIONS_MAIN_DB_USER__, __DVERSIONS_MAIN_DB_PASSWORD__, __DVERSIONS_MAIN_DB_NAME__, __DVERSIONS_MAIN_DB_PORT__);

if ($mysqli->connect_errno) {

	$error[] = "Failed to connect to MySQL: (".$mysqli->connect_errno.") ".$mysqli->connect_error;
}

switch ($_INPUT['mode']) {

	case 'check_syntax':

		$sandbox_mysqli = new mysqli(__SANDBOX_DB_HOST__, __SANDBOX_DB_USER__, __SANDBOX_DB_PASSWORD__, __SANDBOX_DB_NAME__, __SANDBOX_DB_PORT__);

		$sandbox = new Sandbox($mysqli, $sandbox_mysqli);

		if (!$sandbox->isValidSyntax($_INPUT['project_id'], $_INPUT['code'])) {

			$error[] = 'Not valid syntax!';
		}

		break;

	case 'save':

		$version = new Version($mysqli);

		$version->setProperties($_INPUT['project_id'], $_INPUT['description'], $_INPUT['code']);

		if (!$version->save()) {

			$error[] = 'Can\'t save changes';
		}

		break;

	case 'list_versions':

		if (!empty($_INPUT['project_id'])) {

			$stmt = $mysqli->prepare('SELECT v.id, v.description, v.last_updated, u.name FROM version v INNER JOIN user u ON (v.user_id=u.id)  WHERE project_id = ?');
			$stmt->bind_param('i', $_INPUT['project_id']);

			if ($stmt->execute()) {

				$stmt->bind_result($id, $description, $date, $user_name);

				while ($stmt->fetch()) {

					$data[] = array('id'          => $id,
					                'description' => $description,
					                'date'        => $date,
					                'user'        => $user_name
					);
				}

				$stmt->close();
			}

		} else {

			$error[] = 'Project id not specified!';
		}

		break;
	case 'version_info':

		if (!empty($_INPUT['id'])) {

			$version = new Version($mysqli);
			$version->load($_INPUT['id']);
			$data = $version->getProperties();

		} else {

			$error[] = 'Version id not specified!';
		}

		break;

	default:
		header($_SERVER['SERVER_PROTOCOL'].' 400 Bad Request');
		break;
}

$return = array('data' => $data);

if (!empty($error)) {
	$return['error'] = $error;
}

if (!empty($message)) {
	$return['message'] = $message;
}

header('Content-type: '.(isset($_SERVER['HTTP_ACCEPT']) && (strpos($_SERVER['HTTP_ACCEPT'], 'application/json') !== false) ? 'application/json' : 'text/plain'));

echo json_encode($return);