<?php

class DatabaseSchema {

	protected $db;

	protected $database_name;

	protected $object_types = array();

	public function __construct(mysqli $db) {

		$this->db = $db;

		if ($result = $this->db->query("SELECT DATABASE()")) {

			$row = $result->fetch_row();
			$this->database_name = $row[0];

			$this->object_types = array(
				'table'     => array(
					'get_objects_query'       => 'SHOW FULL TABLES WHERE Table_type = \'BASE TABLE\'',
					'get_objects_index'       => 0,
					'get_object_schema_index' => 1
				),
				'view'      => array(
					'get_objects_query'       => 'SHOW FULL TABLES WHERE Table_type = \'VIEW\'',
					'get_objects_index'       => 0,
					'get_object_schema_index' => 1
				),
				'trigger'   => array(
					'get_objects_query'       => 'SHOW TRIGGERS',
					'get_objects_index'       => 0,
					'get_object_schema_index' => 2
				),
				'procedure' => array(
					'get_objects_query'       => 'SHOW PROCEDURE STATUS WHERE Db = \''.$this->database_name.'\'',
					'get_objects_index'       => 1,
					'get_object_schema_index' => 2
				),
				'function'  => array(
					'get_objects_query'       => 'SHOW FUNCTION STATUS WHERE Db = \''.$this->database_name.'\'',
					'get_objects_index'       => 1,
					'get_object_schema_index' => 2
				)
			);

		} else {
			throw new Exception('Database not set.');
		}
	}

	public function getSchema() {

		$schema = array();

		foreach (array_keys($this->object_types) as $object_type) {

			$objects = $this->getObjects($object_type);

			foreach ($objects as $object) {
				$schema[] = $this->getObjectSchema($object, $object_type);
			}
		}

		return implode("\n\n", $schema);

	}

	protected function getObjects($object_type) {

		$object_type = strtolower($object_type);

		$return = array();

		if (!in_array($object_type, array_keys($this->object_types))) {
			throw new Exception('Type '.$object_type.' of database objects not supported.');
		}

		if ($result = $this->db->query($this->object_types[$object_type]['get_objects_query'])) {

			while ($row = $result->fetch_row()) {
				$return[] = $row[$this->object_types[$object_type]['get_objects_index']];
			}

			$result->close();
		}

		return $return;
	}

	protected function getObjectSchema($object_name, $object_type) {

		$object_type = strtolower($object_type);

		if (!in_array($object_type, array_keys($this->object_types))) {
			throw new Exception('Type '.$object_type.' of database objects not supported.');
		}

		if ($result = $this->db->query('SHOW CREATE '.strtoupper($object_type).' `'.$object_name.'`')) {

			$row = $result->fetch_row();
			$return = $row[$this->object_types[$object_type]['get_object_schema_index']].';';

		} else {
			throw new Exception('Object '.$object_name.' of type '.$object_type.' not found in database.');
		}

		if (strtolower($object_type) == 'table') {
			$return = preg_replace("/\s?AUTO_INCREMENT=\d+\s?/", " ", $return);
		}

		return $return;
	}

}